#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <syslog.h>
#include <errno.h>

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <linux/if.h>

#include <linux/if_tun.h>

#include <netinet/in.h>
#include <arpa/inet.h>

int tun_open(char *dev)
{
	struct ifreq ifr;
	int fd;

	if ((fd = open("/dev/net/tun", O_RDWR)) < 0)
		goto failed;

	memset(&ifr, 0, sizeof(ifr));
	ifr.ifr_flags = IFF_TUN | IFF_NO_PI;
	if (*dev)
		strncpy(ifr.ifr_name, dev, IFNAMSIZ);

	if (ioctl(fd, TUNSETIFF, (void *) &ifr) < 0)
		goto failed;

	strcpy(dev, ifr.ifr_name);
	return fd;

failed:
	close(fd);
	return -1;
}

int tun_up(const char *dev, const char * local, const char * remote, int mtu)
{
	struct ifreq ifr;
	int fd = socket(PF_INET, SOCK_DGRAM, 0);

	if(fd < 0)
		return -1;

	memset(&ifr, 0, sizeof(ifr));
	strncpy(ifr.ifr_name, dev, IFNAMSIZ);
	if (ioctl(fd, SIOCGIFFLAGS, &ifr) < 0) 
		goto failed;
	ifr.ifr_flags |= (IFF_UP | IFF_RUNNING);
	ifr.ifr_flags |= IFF_UP;
	if (ioctl(fd, SIOCSIFFLAGS, (void *) &ifr) < 0)
		goto failed;
	memset(&ifr.ifr_ifru, 0, sizeof(ifr.ifr_ifru));
	ifr.ifr_addr.sa_family=AF_INET;
	inet_aton(local, &((struct sockaddr_in *) &ifr.ifr_addr)->sin_addr);
	if (ioctl(fd, SIOCSIFADDR, (void *) &ifr) < 0)
		goto failed;
	memset(&ifr.ifr_ifru, 0, sizeof(ifr.ifr_ifru));
	ifr.ifr_dstaddr.sa_family=AF_INET;
	inet_aton(remote, &((struct sockaddr_in *) &ifr.ifr_dstaddr)->sin_addr);
	if (ioctl(fd, SIOCSIFDSTADDR, (void *) &ifr) < 0)
		goto failed;

	memset(&ifr.ifr_ifru, 0, sizeof(ifr.ifr_ifru));
	ifr.ifr_mtu = mtu;
	if (ioctl(fd, SIOCSIFMTU, (void *) &ifr) < 0)
		goto failed;

	close(fd);
	return 0;

failed:
	close(fd);
	return -1;
}
